import { world, Player, Vector, EntityDamageCause } from "@minecraft/server";

const bulletDamage = {
  // citori_725 bullets
  bullet1: {
    damageBase: 13,
    damageMin: 8,
    fallOffMin: 8,
    fallOffMax: 16,
  },
  // p90 bullets
  bullet2: {
    damageBase: 11,
    damageMin: 5,
    fallOffMin: 13,
    fallOffMax: 26,
  },
  // ak bullets
  bullet3: {
    damageBase: 13,
    damageMin: 8,
    fallOffMin: 24,
    fallOffMax: 48,
  },
  // p250 bullets
  bullet4: {
    damageBase: 9,
    damageMin: 3,
    fallOffMin: 13,
    fallOffMax: 26,
  },
  // m4 bullets
  bullet5: {
    damageBase: 12,
    damageMin: 6,
    fallOffMin: 24,
    fallOffMax: 48,
  },
  // mini-gun bullets
  bullet6: {
    damageBase: 15,
    damageMin: 8,
    fallOffMin: 24,
    fallOffMax: 48,
  },
};

world.events.projectileHit.subscribe((ev) => {
  const src = ev.source;
  if (!(src instanceof Player)) return;

  const proj = ev.projectile;
  if (proj.typeId != "ra:bullet.projectile") return;

  const entityHit = ev.getEntityHit();
  if (!entityHit) return;

  const entity = entityHit.entity;
  /** @type {number} */
  const variant = proj.getComponent("minecraft:mark_variant")?.value ?? 0;
  if (variant == 0) return;
  const damageDef = bulletDamage[`bullet${variant}`];
  if (!damageDef) {
    console.warn(`Could not find a bullet damage definition for variant ${variant}`);
    return;
  }

  const dist = Vector.subtract(src.location, ev.location).length();

  let dmg = damageDef.damageBase;
  if (dist > damageDef.fallOffMin) {
    const fallOff = Math.max(1 - (dist - damageDef.fallOffMin) / (damageDef.fallOffMax - damageDef.fallOffMin), 0);
    dmg = Math.round(Math.max(dmg * fallOff, damageDef.damageMin));
  }
  entity.applyDamage(dmg, {
    cause: EntityDamageCause.projectile,
    damagingProjectile: proj,
  });
  // world.sendMessage(`Damage: ${dmg}`);
});
