import { world, MolangVariableMap } from "@minecraft/server";
import { playSound } from "./utils";

const entityParticles = {
  // Disabled
  chest_minecart: [],
  minecart: [],
  command_block_minecart: [],
  hopper_minecart: [],
  tnt_minecart: [],
  ender_crystal: [],
  armor_stand: [],
  boat: [],
  balloon: [],
  chest_boat: [],
  magma_cube: [],
  "ra:menu_holder": [],
  "ra:ak_47_clip_golden": [],
  "ra:ak_47_clip": [],
  "ra:mini_gun_clip": [],
  "ra:shotgun_shell": [],
  "ra:sniper_casing": [],
  "ra:thunder_gun_clip": [],
  // Custom 
  iron_golem: ["ra:bullet.impact_1", "ra:bullet.impact_sparks", "ra:bullet.impact_sparks", "ra:bullet.impact_small"],
  blaze: ["ra:bullet.impact_sparks", "ra:bullet.impact_small"],
  shulker: [],
  vex: [],
  snow_golem: [],
  slime: ["ra:spider.hurt_blood", "ra:spider.hurt_bits", "ra:bullet.impact_spider"],
  cave_spider: ["ra:spider.hurt_blood", "ra:spider.hurt_bits", "ra:bullet.impact_spider"],
  spider: ["ra:spider.hurt_blood", "ra:spider.hurt_bits", "ra:bullet.impact_spider"],
  warden: [],
  skeleton_horse: ["ra:skeleton.spine_hit", "ra:skeleton.spine_hit.1"],
  skeleton: ["ra:skeleton.spine_hit", "ra:skeleton.spine_hit.1"],
  wither_skeleton: [],
  wither: [],
  stray: [],
  pig: ["ra:hurt_blood", "ra:hurt_blood", "ra:zombie.hurt_bits", "ra:bullet.impact_zombie", "ra:bullet.impact_zombie"],
  cow: ["ra:hurt_blood", "ra:hurt_blood", "ra:zombie.hurt_bits", "ra:bullet.impact_zombie", "ra:bullet.impact_zombie"],
  "ra:target": ["ra:bullet.impact_1", "ra:bullet.impact_sparks"],
  "ra:target_game": ["ra:bullet.impact_1", "ra:bullet.impact_sparks"],
  "ra:z_game": ["ra:hurt_blood", "ra:bullet.impact_zombie", "ra:bullet.impact_1", "ra:bullet.impact_sparks"],
  // Default 
  default: ["ra:hurt_blood", "ra:zombie.hurt_bits", "ra:bullet.impact_zombie"],
};

const sounds = {
  // Disabled
  chest_minecart: [],
  minecart: [],
  command_block_minecart: [],
  hopper_minecart: [],
  tnt_minecart: [],
  ender_crystal: [],
  armor_stand: [],
  boat: [],
  chest_boat: [],
  iron_golem: [],
  slime: [],
  magma_cube: [],
  // Custom sounds
  blaze: [],
  shulker: [],
  balloon: [],
  vex: [],
  snow_golem: [],
  cave_spider: [{ id: "splat_impact", pitch: 1.5, volume: 0.45 }],
  spider: [{ id: "splat_impact", pitch: 1.5, volume: 0.45 }],
  warden: [],
  skeleton_horse: [[{ id: "hit.bone_block", pitch: 0.75, volume: 0.65 }, { id: "hit.bone_block", pitch: 0.25, volume: 0.45 }]],
  skeleton: [[{ id: "hit.bone_block", pitch: 0.75, volume: 0.65 }, { id: "hit.bone_block", pitch: 0.25, volume: 0.45 }]],
  wither_skeleton: [[{ id: "hit.bone_block", pitch: 0.75, volume: 0.65 }, { id: "hit.bone_block", pitch: 0.25, volume: 0.45 }]],
  wither: [],
  stray: [],
  "ra:target": [],
  // Default sounds
  default: [
    { id: "splat_impact", pitch: 1.5, volume: 0.45 },
    [
      { id: "hit.bone_block", pitch: 0.75, volume: 0.65 },
      { id: "hit.bone_block", pitch: 0.25, volume: 0.45 },
    ],
  ],
};

world.events.projectileHit.subscribe((ev) => {
  const proj = ev.projectile;
  if (proj.typeId != "ra:bullet.projectile") return;

  const entityHit = ev.getEntityHit();
  if (!entityHit) return;

  const entity = entityHit.entity;
  const entId = entity.typeId.replace("minecraft:", "");

  const entSounds = sounds[entId] ?? sounds.default;

  const hitLoc = ev.location;
  const dim = ev.dimension;

  const sound = entSounds[Math.floor(Math.random() * entSounds.length)];
  if (Array.isArray(sound)) {
    for (const s of sound) {
      playSound(
        dim,
        s.id,
        Object.assign(s, {
          location: hitLoc,
        })
      );
    }
  } else if (sound) {
    playSound(
      dim,
      sound.id,
      Object.assign(sound, {
        location: hitLoc,
      })
    );
  }

  const entParticles = entityParticles[entId] ?? entityParticles.default;

  const vars = new MolangVariableMap();
  for (const p of entParticles) {
    dim.spawnParticle(p, hitLoc, vars);
  }
});
